#include <cppunit/extensions/HelperMacros.h>
#include "cppUnitHelper.hxx"
#include "similarityHelper.hxx"
#include "FFT_rfftw.hxx"
#include "FFT_numrec.hxx"
#include "FFT_ooura.hxx"
#include "Audio.hxx"
#include "Spectrum.hxx"
#include "XMLStorage.hxx"

namespace CLAMTest
{
	class FFTFunctionalTest;

	CPPUNIT_TEST_SUITE_REGISTRATION( FFTFunctionalTest );

	class FFTFunctionalTest
		: public CppUnit::TestFixture
	{
		CPPUNIT_TEST_SUITE( FFTFunctionalTest );
		CPPUNIT_TEST( test_FFTW_WithPowerOfTwoInput );
		CPPUNIT_TEST( test_NumRec_WithPowerOfTwoInput );
		CPPUNIT_TEST( test_Ooura_WithPowerOfTwoInput );
		CPPUNIT_TEST( test_FFTW_WithNonPowerOfTwoInput );
		CPPUNIT_TEST( test_NumRec_WithNonPowerOfTwoInput );
		CPPUNIT_TEST( test_Ooura_WithNonPowerOfTwoInput );
		CPPUNIT_TEST( test_NumRec_WithComplex );
		CPPUNIT_TEST( test_NumRec_WithPolar );
		CPPUNIT_TEST( test_NumRec_WithBPF );
		CPPUNIT_TEST( test_Ooura_WithComplex );
		CPPUNIT_TEST( test_Ooura_WithPolar );
		CPPUNIT_TEST( test_Ooura_WithBPF );
		CPPUNIT_TEST( test_FFTW_WithComplex );
		CPPUNIT_TEST( test_FFTW_WithPolar );
		CPPUNIT_TEST( test_FFTW_WithBPF );
		CPPUNIT_TEST_SUITE_END();

	protected:

		static double         smEqualityThreshold;
		static bool           smBack2BackDataLoaded;
		static CLAM::Spectrum smReferenceP2Spectrum;
		static CLAM::Spectrum smReferenceNP2Spectrum;

		std::string mPathToTestData;

		CLAM::FFT_base& createFFTObject( const std::string& implementation )
		{
			if ( implementation == "FFTW" )
			{
				static CLAM::FFT_rfftw obj;

				return obj;
			}
			else if ( implementation == "NumRec" )
			{
				static CLAM::FFT_numrec obj;
				
				return obj;
			}
			else if ( implementation == "Ooura" )
			{
				static CLAM::FFT_ooura obj;

				return obj;
			}
			else
				return *(CLAM::FFT_base*)NULL;
		}

		void setupSine_F0400Hz_SR8kHz( CLAM::Audio& audio, int nSamples ) 
		{
			const CLAM::TSize sampleRate = 8000;
			const CLAM::TData sineFreq = 400.0f;

			audio.SetSize( nSamples );
			audio.SetSampleRate( sampleRate );
			audio.SetBeginTime( CLAM::TTime(0.0) );

			for ( CLAM::TSize i=0; i<nSamples; i++ )
				audio.GetBuffer()[i]=0.625+0.5*sin(2.0*sineFreq*400.0*(((float)i)/sampleRate));
		}

		void setupSine_F0400Hz_SR8kHz_1024samples( CLAM::Audio& audio )
		{
			setupSine_F0400Hz_SR8kHz(audio, 1024);
		}

		void setupSine_F0400Hz_SR8kHz_884samples( CLAM::Audio& audio )
		{
			setupSine_F0400Hz_SR8kHz(audio, 884);
		}

		void setupSpectrumToStoreFFTOutput( CLAM::Spectrum& spectrum )
		{
			CLAM::SpecTypeFlags specFlags;
			specFlags.bComplex = 1;
			specFlags.bMagPhase = 1;
			specFlags.bPolar = 1;
			specFlags.bMagPhaseBPF = 1;

			spectrum.SetType( specFlags );
		}

		static void loadBack2BackDataset( const std::string& pathToTestData )
		{
			if ( !smBack2BackDataLoaded )
			{

				CLAM::XMLStorage::Restore( smReferenceP2Spectrum,
							   pathToTestData + "OneSineSpectrum_RectWindow_P2.xml" );
				CLAM::XMLStorage::Restore( smReferenceNP2Spectrum,
							   pathToTestData + "OneSineSpectrum_RectWindow_NP2.xml" );

				smBack2BackDataLoaded = true;
			}
		}

	public:
		void setUp()
		{
			mPathToTestData = GetTestDataDirectory("spectralData/");
			loadBack2BackDataset( mPathToTestData );
		}

		void tearDown()
		{
		}

	private:
		
		void test_FFTW_WithPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "FFTW" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );
			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );
			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );
		}
		
		void test_NumRec_WithBPF()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bMagPhaseBPF=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "NumRec" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			
			processing.Configure( processingConfig );
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSize() 
					== output.GetSize() );
			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			

			
		}

		void test_NumRec_WithPolar()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bPolar=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "NumRec" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}
		void test_NumRec_WithComplex()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bComplex=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "NumRec" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}
		
		void test_NumRec_WithPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "NumRec" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}

		void test_Ooura_WithBPF()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bMagPhaseBPF=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "Ooura" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSize() 
					== output.GetSize() );
			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			

			
		}

		void test_Ooura_WithPolar()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bPolar=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "Ooura" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}
		void test_Ooura_WithComplex()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bComplex=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "Ooura" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}
		

		void test_Ooura_WithPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "Ooura" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );


		}

		void test_FFTW_WithBPF()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bMagPhaseBPF=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "FFTW" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );
			//setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSize() 
					== output.GetSize() );
			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			

			
		}

		void test_FFTW_WithPolar()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bPolar=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "FFTW" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}
		void test_FFTW_WithComplex()
		{
			CLAM::Audio     input;
			CLAM::SpecTypeFlags flg;
			flg.bMagPhase=0;
			flg.bComplex=1;
			CLAM::Spectrum  output;
			output.SetType(flg);

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "FFTW" );

			setupSine_F0400Hz_SR8kHz_1024samples( input );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do(input,output);
			processing.Stop();

			flg.bMagPhase=1;
			output.SetTypeSynchronize(flg);
			double similarity = evaluateSimilarity( smReferenceP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}

		void test_FFTW_WithNonPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "FFTW" );

			setupSine_F0400Hz_SR8kHz_884samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			processing.Configure( processingConfig );
			
			processing.Start();
			processing.Do( input, output );
			processing.Stop();

			double similarity = evaluateSimilarity( smReferenceNP2Spectrum.GetMagBuffer(),
								output.GetMagBuffer() );

			CPPUNIT_ASSERT( smEqualityThreshold <= similarity );
			CPPUNIT_ASSERT( smReferenceP2Spectrum.GetSpectralRange() 
					== output.GetSpectralRange() );

			
		}

		void test_NumRec_WithNonPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "NumRec" );

			setupSine_F0400Hz_SR8kHz_884samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			CPPUNIT_ASSERT_EQUAL( false, processing.Configure( processingConfig ) );
			
		}

		void test_Ooura_WithNonPowerOfTwoInput()
		{
			CLAM::Audio     input;
			CLAM::Spectrum  output;

			CLAM::FFTConfig processingConfig;
			CLAM::FFT_base&      processing = createFFTObject( "Ooura" );

			setupSine_F0400Hz_SR8kHz_884samples( input );
			setupSpectrumToStoreFFTOutput( output );

			output.SetSize( CLAM::TSize(input.GetSize()/2 + 1) );

			processingConfig.SetAudioSize( input.GetSize() );

			CPPUNIT_ASSERT_EQUAL( false, processing.Configure( processingConfig ) );	
		}
	};

	double FFTFunctionalTest::smEqualityThreshold = 0.9999;
	bool FFTFunctionalTest::smBack2BackDataLoaded = false;
	CLAM::Spectrum FFTFunctionalTest::smReferenceP2Spectrum;
	CLAM::Spectrum FFTFunctionalTest::smReferenceNP2Spectrum;
}
